@@ -230,16 +230,19 @@ my @hash_types =
   13751,
   13761,
   13771,
+  13781,
   13800,
   13900,
   14000,
   14100,
   14400,
+  14500,
   14700,
   14800,
   14900,
   15000,
   15100,
+  15200,
   15300,
   15400,
   15500,
@@ -250,10 +253,13 @@ my @hash_types =
   16200,
   16300,
   16400,
+  16500,
   16600,
+  16700,
   16800,
   16801,
   16900,
+  17210,
   17300,
   17400,
   17500,
@@ -333,12 +339,23 @@ my @hash_types =
   24700,
   24800,
   24900,
+  25000,
+  25100,
+  25200,
   25300,
   25400,
   25500,
+  25700,
   25900,
   26000,
   26100,
+  26200,
+  26300,
+  26401,
+  26402,
+  26403,
+  26500,
+  26600,
 );
 
 if (scalar @ARGV)
@@ -392,7 +409,6 @@ for my $hash_type (@hash_types)
     "--wordlist-autohex-disable",
     "--potfile-disable",
     "--logfile-disable",
-    "--hwmon-disable",
     "--status",
     "--status-timer", 1,
     "--runtime", $runtime,
